"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UIBoolean = exports.UIPoints3 = exports.UIPoints2 = exports.UIPoints = exports.UIOutliner = exports.UICubeTexture = exports.UITexture = void 0;
/* eslint-disable no-continue */
/* eslint-disable max-classes-per-file */
const THREE = __importStar(require("three"));
const RGBELoader_1 = require("../../../examples/jsm/loaders/RGBELoader");
const TGALoader_1 = require("../../../examples/jsm/loaders/TGALoader");
const ui_1 = require("./ui");
const MoveObjectCommand_1 = require("../commands/MoveObjectCommand");
class UITexture extends ui_1.UISpan {
    constructor(mapping) {
        super();
        const scope = this;
        const form = document.createElement('form');
        const input = document.createElement('input');
        input.type = 'file';
        input.addEventListener('change', (event) => {
            loadFile(event.target.files[0]);
        });
        form.appendChild(input);
        const canvas = document.createElement('canvas');
        canvas.width = 32;
        canvas.height = 16;
        canvas.style.cursor = 'pointer';
        canvas.style.marginRight = '5px';
        canvas.style.border = '1px solid #888';
        canvas.addEventListener('click', () => {
            input.click();
        });
        canvas.addEventListener('drop', (event) => {
            event.preventDefault();
            event.stopPropagation();
            loadFile(event.dataTransfer.files[0]);
        });
        this.dom.appendChild(canvas);
        function loadFile(file) {
            const extension = file.name.split('.').pop().toLowerCase();
            const reader = new FileReader();
            if (extension === 'hdr' || extension === 'pic') {
                reader.addEventListener('load', (event) => {
                    // assuming RGBE/Radiance HDR iamge format
                    const loader = new RGBELoader_1.RGBELoader();
                    loader.load(event.target.result, (hdrTexture) => {
                        hdrTexture.sourceFile = file.name;
                        hdrTexture.isHDRTexture = true;
                        scope.setValue(hdrTexture);
                        if (scope.onChangeCallback)
                            scope.onChangeCallback(hdrTexture);
                    });
                });
                reader.readAsDataURL(file);
            }
            else if (extension === 'tga') {
                reader.addEventListener('load', (event) => {
                    const canvas = new TGALoader_1.TGALoader().parse(event.target.result);
                    const texture = new THREE.CanvasTexture(canvas, mapping);
                    texture.sourceFile = file.name;
                    scope.setValue(texture);
                    if (scope.onChangeCallback)
                        scope.onChangeCallback(texture);
                }, false);
                reader.readAsArrayBuffer(file);
            }
            else if (file.type.match('image.*')) {
                reader.addEventListener('load', (event) => {
                    const image = document.createElement('img');
                    image.addEventListener('load', () => {
                        const texture = new THREE.Texture(this, mapping);
                        texture.sourceFile = file.name;
                        texture.needsUpdate = true;
                        scope.setValue(texture);
                        if (scope.onChangeCallback)
                            scope.onChangeCallback(texture);
                    }, false);
                    image.src = event.target.result;
                }, false);
                reader.readAsDataURL(file);
            }
            form.reset();
        }
        this.texture = null;
        this.onChangeCallback = null;
    }
    getValue() {
        return this.texture;
    }
    setValue(texture) {
        const canvas = this.dom.children[0];
        const context = canvas.getContext('2d');
        // Seems like context can be null if the canvas is not visible
        if (context) {
            // Always clear the context before set new texture, because new texture may has transparency
            context.clearRect(0, 0, canvas.width, canvas.height);
        }
        if (texture !== null) {
            const { image } = texture;
            if (image !== undefined && image.width > 0) {
                canvas.title = texture.sourceFile;
                const scale = canvas.width / image.width;
                if (image.data === undefined) {
                    context.drawImage(image, 0, 0, image.width * scale, image.height * scale);
                }
                else {
                    const canvas2 = renderToCanvas(texture);
                    context.drawImage(canvas2, 0, 0, image.width * scale, image.height * scale);
                }
            }
            else {
                canvas.title = `${texture.sourceFile} (error)`;
            }
        }
        else {
            canvas.title = 'empty';
        }
        this.texture = texture;
    }
    setEncoding(encoding) {
        const texture = this.getValue();
        if (texture !== null) {
            texture.encoding = encoding;
        }
        return this;
    }
    onChange(callback) {
        this.onChangeCallback = callback;
        return this;
    }
}
exports.UITexture = UITexture;
class UICubeTexture extends ui_1.UIElement {
    constructor() {
        const container = new ui_1.UIDiv();
        super(container.dom);
        this.cubeTexture = null;
        this.onChangeCallback = null;
        this.textures = [];
        const scope = this;
        const pRow = new ui_1.UIRow();
        const nRow = new ui_1.UIRow();
        pRow.add(new ui_1.UIText('P:').setWidth('35px'));
        nRow.add(new ui_1.UIText('N:').setWidth('35px'));
        const posXTexture = new UITexture().onChange(onTextureChanged);
        const negXTexture = new UITexture().onChange(onTextureChanged);
        const posYTexture = new UITexture().onChange(onTextureChanged);
        const negYTexture = new UITexture().onChange(onTextureChanged);
        const posZTexture = new UITexture().onChange(onTextureChanged);
        const negZTexture = new UITexture().onChange(onTextureChanged);
        this.textures.push(posXTexture, negXTexture, posYTexture, negYTexture, posZTexture, negZTexture);
        pRow.add(posXTexture);
        pRow.add(posYTexture);
        pRow.add(posZTexture);
        nRow.add(negXTexture);
        nRow.add(negYTexture);
        nRow.add(negZTexture);
        container.add(pRow, nRow);
        function onTextureChanged() {
            const images = [];
            for (let i = 0; i < scope.textures.length; i += 1) {
                const texture = scope.textures[i].getValue();
                if (texture !== null) {
                    images.push(texture.isHDRTexture ? texture : texture.image);
                }
            }
            if (images.length === 6) {
                const cubeTexture = new THREE.CubeTexture(images);
                cubeTexture.needsUpdate = true;
                if (images[0].isHDRTexture)
                    cubeTexture.isHDRTexture = true;
                scope.cubeTexture = cubeTexture;
                if (scope.onChangeCallback)
                    scope.onChangeCallback(cubeTexture);
            }
        }
    }
    setEncoding(encoding) {
        const cubeTexture = this.getValue();
        if (cubeTexture !== null) {
            cubeTexture.encoding = encoding;
        }
        return this;
    }
    getValue() {
        return this.cubeTexture;
    }
    setValue(cubeTexture) {
        this.cubeTexture = cubeTexture;
        if (cubeTexture !== null) {
            const images = cubeTexture.image;
            if (Array.isArray(images) === true && images.length === 6) {
                for (let i = 0; i < images.length; i += 1) {
                    const image = images[i];
                    const texture = new THREE.Texture(image);
                    this.textures[i].setValue(texture);
                }
            }
        }
        else {
            const { textures } = this;
            for (let i = 0; i < textures.length; i += 1) {
                textures[i].setValue(null);
            }
        }
        return this;
    }
    onChange(callback) {
        this.onChangeCallback = callback;
        return this;
    }
}
exports.UICubeTexture = UICubeTexture;
class UIOutliner extends ui_1.UIDiv {
    constructor(editor) {
        super();
        this.dom.className = 'Outliner';
        this.dom.tabIndex = 0; // keyup event is ignored without setting tabIndex
        const scope = this;
        // hack
        this.scene = editor.scene;
        // Prevent native scroll behavior
        this.dom.addEventListener('keydown', (event) => {
            switch (event.keyCode) {
                case 38: // up
                case 40: // down
                    event.preventDefault();
                    event.stopPropagation();
                    break;
                default:
                    break;
            }
        });
        // Keybindings to support arrow navigation
        this.dom.addEventListener('keyup', (event) => {
            switch (event.keyCode) {
                case 38: // up
                    scope.selectIndex(scope.selectedIndex - 1);
                    break;
                case 40: // down
                    scope.selectIndex(scope.selectedIndex + 1);
                    break;
                default:
                    break;
            }
        });
        this.editor = editor;
        this.options = [];
        this.selectedIndex = -1;
        this.selectedValue = null;
    }
    selectIndex(index) {
        if (index >= 0 && index < this.options.length) {
            this.setValue(this.options[index].value);
            const changeEvent = document.createEvent('HTMLEvents');
            changeEvent.initEvent('change', true, true);
            this.dom.dispatchEvent(changeEvent);
        }
    }
    setOptions(options) {
        const scope = this;
        while (scope.dom.children.length > 0) {
            scope.dom.removeChild(scope.dom.firstChild);
        }
        function onClick() {
            scope.setValue(this.value);
            const changeEvent = document.createEvent('HTMLEvents');
            changeEvent.initEvent('change', true, true);
            scope.dom.dispatchEvent(changeEvent);
        }
        // Drag
        let currentDrag;
        function onDrag() {
            currentDrag = this;
        }
        function onDragStart(event) {
            event.dataTransfer.setData('text', 'foo');
        }
        function onDragOver(event) {
            if (this === currentDrag)
                return;
            const area = event.offsetY / this.clientHeight;
            if (area < 0.25) {
                this.className = 'option dragTop';
            }
            else if (area > 0.75) {
                this.className = 'option dragBottom';
            }
            else {
                this.className = 'option drag';
            }
        }
        function onDragLeave() {
            if (this === currentDrag)
                return;
            this.className = 'option';
        }
        function onDrop(event) {
            if (this === currentDrag || currentDrag === undefined)
                return;
            this.className = 'option';
            const { scene } = scope;
            const object = scene.getObjectById(currentDrag.value);
            const area = event.offsetY / this.clientHeight;
            if (area < 0.25) {
                const nextObject = scene.getObjectById(this.value);
                moveObject(object, nextObject.parent, nextObject);
            }
            else if (area > 0.75) {
                let nextObject;
                let parent;
                if (this.nextSibling !== null) {
                    nextObject = scene.getObjectById(this.nextSibling.value);
                    parent = nextObject.parent;
                }
                else {
                    // end of list (no next object)
                    nextObject = null;
                    parent = scene.getObjectById(this.value).parent;
                }
                moveObject(object, parent, nextObject);
            }
            else {
                const parentObject = scene.getObjectById(this.value);
                moveObject(object, parentObject);
            }
        }
        function moveObject(object, newParent, nextObject) {
            if (nextObject === null)
                nextObject = undefined;
            let newParentIsChild = false;
            object.traverse((child) => {
                if (child === newParent)
                    newParentIsChild = true;
            });
            if (newParentIsChild)
                return;
            const { editor } = scope;
            editor.execute(new MoveObjectCommand_1.MoveObjectCommand(editor, object, newParent, nextObject));
            const changeEvent = document.createEvent('HTMLEvents');
            changeEvent.initEvent('change', true, true);
            scope.dom.dispatchEvent(changeEvent);
        }
        //
        scope.options = [];
        for (let i = 0; i < options.length; i += 1) {
            const div = options[i];
            div.className = 'option';
            scope.dom.appendChild(div);
            scope.options.push(div);
            div.addEventListener('click', onClick);
            if (div.draggable === true) {
                div.addEventListener('drag', onDrag);
                div.addEventListener('dragstart', onDragStart); // Firefox needs this
                div.addEventListener('dragover', onDragOver);
                div.addEventListener('dragleave', onDragLeave);
                div.addEventListener('drop', onDrop);
            }
        }
        return scope;
    }
    getValue() {
        return this.selectedValue;
    }
    setValue(value) {
        for (let i = 0; i < this.options.length; i += 1) {
            const element = this.options[i];
            if (element.value === value) {
                element.classList.add('active');
                // scroll into view
                const y = element.offsetTop - this.dom.offsetTop;
                const bottomY = y + element.offsetHeight;
                const minScroll = bottomY - this.dom.offsetHeight;
                if (this.dom.scrollTop > y) {
                    this.dom.scrollTop = y;
                }
                else if (this.dom.scrollTop < minScroll) {
                    this.dom.scrollTop = minScroll;
                }
                this.selectedIndex = i;
            }
            else {
                element.classList.remove('active');
            }
        }
        this.selectedValue = value;
        return this;
    }
}
exports.UIOutliner = UIOutliner;
class UIPoints extends ui_1.UISpan {
    constructor() {
        super();
        this.dom.style.display = 'inline-block';
        this.pointsList = new ui_1.UIDiv();
        this.add(this.pointsList);
        this.pointsUI = [];
        this.lastPointIdx = 0;
        this.onChangeCallback = null;
        // TODO Remove this bind() stuff
        // this.update = () =>  {
        //   if (this.onChangeCallback !== null) {
        //     this.onChangeCallback();
        //   }
        // }.bind(this);
    }
    onChange(callback) {
        this.onChangeCallback = callback;
        return this;
    }
    clear() {
        for (let i = 0; i < this.pointsUI.length; i += 1) {
            if (this.pointsUI[i]) {
                this.deletePointRow(i, true);
            }
        }
        this.lastPointIdx = 0;
    }
    deletePointRow(idx, dontUpdate) {
        if (!this.pointsUI[idx])
            return;
        this.pointsList.remove(this.pointsUI[idx].row);
        this.pointsUI.splice(idx, 1);
        if (dontUpdate !== true) {
            this.update();
        }
        this.lastPointIdx -= 1;
    }
}
exports.UIPoints = UIPoints;
class UIPoints2 extends UIPoints {
    constructor() {
        super();
        const row = new ui_1.UIRow();
        this.add(row);
        const addPointButton = new ui_1.UIButton('+');
        addPointButton.onClick(() => {
            if (this.pointsUI.length === 0) {
                this.pointsList.add(this.createPointRow(0, 0));
            }
            else {
                const point = this.pointsUI[this.pointsUI.length - 1];
                this.pointsList.add(this.createPointRow(point.x.getValue(), point.y.getValue()));
            }
            this.update();
        });
        row.add(addPointButton);
    }
    getValue() {
        const points = [];
        let count = 0;
        for (let i = 0; i < this.pointsUI.length; i += 1) {
            const pointUI = this.pointsUI[i];
            // eslint-disable-next-line no-continue
            if (!pointUI)
                continue;
            points.push(new THREE.Vector2(pointUI.x.getValue(), pointUI.y.getValue()));
            count += 1;
            pointUI.lbl.setValue(count);
        }
        return points;
    }
    setValue(points) {
        this.clear();
        for (let i = 0; i < points.length; i += 1) {
            const point = points[i];
            this.pointsList.add(this.createPointRow(point.x, point.y));
        }
        this.update();
        return this;
    }
    createPointRow(x, y) {
        const pointRow = new ui_1.UIDiv();
        const lbl = new ui_1.UIText(this.lastPointIdx + 1).setWidth('20px');
        const txtX = new ui_1.UINumber(x).setWidth('30px').onChange(this.update);
        const txtY = new ui_1.UINumber(y).setWidth('30px').onChange(this.update);
        const scope = this;
        const btn = new ui_1.UIButton('-').onClick(() => {
            if (scope.isEditing)
                return;
            const idx = scope.pointsList.getIndexOfChild(pointRow);
            scope.deletePointRow(idx);
        });
        this.pointsUI.push({ row: pointRow, lbl, x: txtX, y: txtY });
        this.lastPointIdx += 1;
        pointRow.add(lbl, txtX, txtY, btn);
        return pointRow;
    }
}
exports.UIPoints2 = UIPoints2;
class UIPoints3 extends UIPoints {
    constructor() {
        super();
        const row = new ui_1.UIRow();
        this.add(row);
        const addPointButton = new ui_1.UIButton('+');
        addPointButton.onClick(() => {
            if (this.pointsUI.length === 0) {
                this.pointsList.add(this.createPointRow(0, 0, 0));
            }
            else {
                const point = this.pointsUI[this.pointsUI.length - 1];
                this.pointsList.add(this.createPointRow(point.x.getValue(), point.y.getValue(), point.z.getValue()));
            }
            this.update();
        });
        row.add(addPointButton);
    }
    getValue() {
        const points = [];
        let count = 0;
        for (let i = 0; i < this.pointsUI.length; i += 1) {
            const pointUI = this.pointsUI[i];
            if (!pointUI)
                continue;
            points.push(new THREE.Vector3(pointUI.x.getValue(), pointUI.y.getValue(), pointUI.z.getValue()));
            count += 1;
            pointUI.lbl.setValue(count);
        }
        return points;
    }
    setValue(points) {
        this.clear();
        for (let i = 0; i < points.length; i += 1) {
            const point = points[i];
            this.pointsList.add(this.createPointRow(point.x, point.y, point.z));
        }
        this.update();
        return this;
    }
    createPointRow(x, y, z) {
        const pointRow = new ui_1.UIDiv();
        const lbl = new ui_1.UIText(this.lastPointIdx + 1).setWidth('20px');
        const txtX = new ui_1.UINumber(x).setWidth('30px').onChange(this.update);
        const txtY = new ui_1.UINumber(y).setWidth('30px').onChange(this.update);
        const txtZ = new ui_1.UINumber(z).setWidth('30px').onChange(this.update);
        const scope = this;
        const btn = new ui_1.UIButton('-').onClick(() => {
            if (scope.isEditing)
                return;
            const idx = scope.pointsList.getIndexOfChild(pointRow);
            scope.deletePointRow(idx);
        });
        this.pointsUI.push({ row: pointRow, lbl, x: txtX, y: txtY, z: txtZ });
        this.lastPointIdx += 1;
        pointRow.add(lbl, txtX, txtY, txtZ, btn);
        return pointRow;
    }
}
exports.UIPoints3 = UIPoints3;
class UIBoolean extends ui_1.UISpan {
    constructor(boolean, text) {
        super();
        this.setMarginRight('4px');
        this.checkbox = new ui_1.UICheckbox(boolean);
        this.text = new ui_1.UIText(text).setMarginLeft('3px');
        this.add(this.checkbox);
        this.add(this.text);
    }
    getValue() {
        return this.checkbox.getValue();
    }
    setValue(value) {
        return this.checkbox.setValue(value);
    }
}
exports.UIBoolean = UIBoolean;
let renderer;
function renderToCanvas(texture) {
    if (renderer === undefined) {
        renderer = new THREE.WebGLRenderer();
        renderer.outputEncoding = THREE.sRGBEncoding;
    }
    const { image } = texture;
    renderer.setSize(image.width, image.height, false);
    const scene = new THREE.Scene();
    const camera = new THREE.OrthographicCamera(-1, 1, 1, -1, 0, 1);
    const material = new THREE.MeshBasicMaterial({ map: texture });
    const quad = new THREE.PlaneGeometry(2, 2);
    const mesh = new THREE.Mesh(quad, material);
    scene.add(mesh);
    renderer.render(scene, camera);
    return renderer.domElement;
}
