"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const NodeFunction_1 = __importDefault(require("../core/NodeFunction"));
const NodeFunctionInput_1 = __importDefault(require("../core/NodeFunctionInput"));
const declarationRegexp = /^[fn]*\s*([a-z_0-9]+)?\s*\(([\s\S]*?)\)\s*[\-\>]*\s*([a-z_0-9]+)?/i;
const propertiesRegexp = /[a-z_0-9]+/gi;
const wgslTypeLib = {
    f32: 'float',
};
const parse = (source) => {
    source = source.trim();
    const declaration = source.match(declarationRegexp);
    if (declaration !== null && declaration.length === 4) {
        // tokenizer
        const inputsCode = declaration[2];
        const propsMatches = [];
        let nameMatch = null;
        while ((nameMatch = propertiesRegexp.exec(inputsCode)) !== null) {
            propsMatches.push(nameMatch);
        }
        // parser
        const inputs = [];
        let i = 0;
        while (i < propsMatches.length) {
            // default
            const name = propsMatches[(i += 1)][0];
            let type = propsMatches[(i += 1)][0];
            type = wgslTypeLib[type] || type;
            // precision
            if (i < propsMatches.length && /^[fui]\d{2}$/.test(propsMatches[i][0]) === true)
                i += 1;
            // add input
            inputs.push(new NodeFunctionInput_1.default(type, name));
        }
        //
        const blockCode = source.substring(declaration[0].length);
        const name = declaration[1] !== undefined ? declaration[1] : '';
        const type = declaration[3] || 'void';
        return {
            type,
            inputs,
            name,
            inputsCode,
            blockCode,
        };
    }
    else {
        throw new Error('FunctionNode: Function is not a WGSL code.');
    }
};
class WGSLNodeFunction extends NodeFunction_1.default {
    constructor(source) {
        const { type, inputs, name, inputsCode, blockCode } = parse(source);
        super(type, inputs, name);
        this.inputsCode = inputsCode;
        this.blockCode = blockCode;
    }
    getCode(name = this.name) {
        const type = this.type !== 'void' ? '-> ' + this.type : '';
        return `fn ${name} ( ${this.inputsCode.trim()} ) ${type}` + this.blockCode;
    }
}
exports.default = WGSLNodeFunction;
